/*
 * @(#)POCServlet.java	1.00 98/02/18 - original
 *                           99/01/10 - updated for WebSphere V2.0
 *                           99/01/10 - parameters taken from init parms
 *
 *  This Servlet is intended to be a prototype/aid when building a proof-of-concept.
 *
 *  See accompanying HTML for Disclaimers and Trademark acknoledgements
 *
 *  This Servlet has not gone through any official IBM testing, and as such, it should
 *  be considered "as-is", and should not be used in a production environment.
 *
 *  This program assumes you are using IBM's Servlet Express
 *
 *  Please see the accompanying HTML files for a description of all the steps necessary
 *     to change this into a proof-of-concept program for you or your customer.
 *
 * This servlet is currently set up for conference enrollment - change is as per 
 *     accompanying HTML documentation
 * 
 */ 

// ###1#
// import statements to include necessary classes
//

import java.io.*;                                             // java classes
import java.util.*;                                           // java classes

import POCDataBean;                                           // for data passing
import java.beans.Beans;                                      // java JavaBeans

import com.ibm.ctg.client.*;                               // IBM CICS Gateway for Java

import javax.servlet.*;                                       // Servlet classes
import javax.servlet.http.*;                                  // Servlet classes




/**
 * This is an example of an HTTP Servlet.  It responds to the GET
 * and HEAD methods of the HTTP protocol.
 */
public class POCServlet extends HttpServlet { 
    //  ###2# 
    //*********************************************************************************************************
    //*********************************************************************************************************
    //*** instance variables  ###3#
    //*********************************************************************************************************
    //*********************************************************************************************************
	JavaGateway CICSGatewayConnection = null;		// the CICS Java Gateway Object
                                                                //   one connection object for everyone
	String sAppName = "POCServlet: ";
	String sConnectErrors = "";

	String strJGateName;					// the CICS Java Gateway location
	int iJGatePort;						// the CICS Java Gateway Port
	String strServerName;					// the target CICS region
	boolean bCheckStatus = true;				// should we check the CICS Client and Server Status?
                                                                //   messages will go to servlet express file
	String strSignProg;					// CICS signon program name, check commarea layout below
	String strRequProg;					// CICS request program name
	ECIRequest eciStatRequest;				// ECI request block for status checks


    //*********************************************************************************************************
    //*********************************************************************************************************
    //***  init is run when the servlet is loaded
    //     Try to connect to the CICS Gateway for Java if it is there
    //*********************************************************************************************************
    //*********************************************************************************************************
    // ###4#
	public void init(ServletConfig config) throws ServletException  {
		super.init(config);				// drive the supplied init method
		System.out.println(sAppName+"=================================================================");
		System.out.println(sAppName+"init() method entered");

		strJGateName = getInitParameter("gateway");
		if (strJGateName == null) {
			System.out.println(sAppName+"gateway initparm missing");
			sConnectErrors = sConnectErrors+"<LI>The gateway initparm is missing, consult documentation</LI>";
		} else {
			System.out.println(sAppName+"The value for gateway is "+strJGateName+"<-");
		}

		try {
			iJGatePort = Integer.parseInt(getInitParameter("port"));
			System.out.println(sAppName+"The value for Port is"+iJGatePort+"<-");
		} catch (NumberFormatException e) {
			System.out.println(sAppName+"port value is missing or invalid");
			sConnectErrors = sConnectErrors+"<LI>The value for port is missing or invalid, consult documentation</LI>";
		}

		strServerName = getInitParameter("server");
		if (strServerName == null) {
			System.out.println(sAppName+"server initparm missing");
			sConnectErrors = sConnectErrors+"<LI>The server initparm is missing, consult documentation</LI>";
		} else {
			System.out.println(sAppName+"The value for server is "+strServerName+"<-");
		}

		strSignProg = getInitParameter("signprog");
		if (strSignProg == null) {
			System.out.println(sAppName+"signprog initparm missing");
			sConnectErrors = sConnectErrors+"<LI>The signprog initparm is missing, consult documentation</LI>";
		} else {
			System.out.println(sAppName+"The value for signprog is "+strSignProg+"<-");
		}

		strRequProg = getInitParameter("requestprog");
		if (strRequProg == null) {
			System.out.println(sAppName+"requestprog initparm missing");
			sConnectErrors = sConnectErrors+"<LI>The requestprog initparm is missing, consult documentation</LI>";
		} else {
			System.out.println(sAppName+"The value for requestprog is "+strRequProg+"<-");
		}

		if (sConnectErrors.equals("")) {  // if we received all of our initparms, this should be null
			connectToGateway();				// open a path to the CICS Gateway for Java
		}
	}  //end init


    /*********************************************************************************/
    /**  attempt a CICS Java Gateway Connection                                      */
    /*********************************************************************************/
    // ###5#
    public boolean connectToGateway() {
        if (CICSGatewayConnection != null) { return true; }
        try {
		CICSGatewayConnection = new JavaGateway(strJGateName, iJGatePort);
		System.out.println(sAppName+"Successfully created JavaGateway Object");
           	/* Find out our general state information */
            	if (bCheckStatus) {
                	System.out.println(sAppName+" --> === General Status ===");
                	displayState(null, null);
	           	/* Find out what systems are available */
        	    	eciStatRequest = ECIRequest.listSystems(10);
            		CICSGatewayConnection.flow(eciStatRequest);
            		System.out.println(sAppName+" --> === Available Servers ===");
            		if (eciStatRequest.getRc() == 0) {
                		for (int iSysLoop = 0; iSysLoop < eciStatRequest.SystemList.size(); iSysLoop += 2) {
                    			if (bCheckStatus) {
                        			displayState(
                            				(String) eciStatRequest.SystemList.elementAt(iSysLoop),
                            				(String) eciStatRequest.SystemList.elementAt(iSysLoop + 1));
    	                		}
        	            		else {
                	        		System.out.println(sAppName+" ----> " + 
						(String) eciStatRequest.SystemList.elementAt(iSysLoop) + ", Description : " +
                            			(String) eciStatRequest.SystemList.elementAt(iSysLoop + 1));
                    			}
                		}
         	   	} else {
	        	        System.out.println(sAppName+"ECIStatRequest.listSystems failed with Rc : " +
                       		    eciStatRequest.getRc());
      		  	        CICSGatewayConnection = null;
				return false;
            		}
		}
            	return true;
	} catch (IOException e) {
                System.out.println(sAppName+"----> Cannot connect to CICS Gateway for Java: "+e);
		CICSGatewayConnection = null;
		return false;
	}


    } //end of connectToGateway

    /*********************************************************************************/
    /**  This method does an ECI_STATE_SYNC call to check the server status          */
    /*********************************************************************************/
    // ###6#
    private void displayState(String strServerName, String strServerDescription) 
                                 throws IOException {
        //System.out.println(" displayState method entered");
        /* Make a getStatus call */
        eciStatRequest = (new ECIRequest()).getStatus(strServerName);
        CICSGatewayConnection.flow(eciStatRequest);
        
        /* Display the relevant information */
        if (strServerName == null) {
            if (eciStatRequest.Cics_Rc != 0) {
                System.out.println(sAppName+" ECI_STATE_SYNC general query failed with return code : " 
                           + eciStatRequest.Cics_Rc);
            }
            else {
                System.out.println(sAppName+" Connection status: " +
                           eciStatRequest.stringConnectionType(eciStatRequest.ConnectionType));
            }
        }
        else {
            System.out.println(sAppName+" " + strServerName +
                       ", Description: " + strServerDescription);
            System.out.println(sAppName+"  Server status: " +
                       eciStatRequest.stringServerStatus(eciStatRequest.CicsServerStatus));
            System.out.println(sAppName+"  Client status: " +
                       eciStatRequest.stringClientStatus(eciStatRequest.CicsClientStatus));

        }
    } // end of displayState()

    //**********************************************************************************************************
    //**********************************************************************************************************
    //*  process the GET method (get processing is the default when the servlet is entered other than by post
    //*   processing
    //**********************************************************************************************************
    //**********************************************************************************************************
    //  ###7#
    public void doGet(HttpServletRequest req, HttpServletResponse res)
	          throws ServletException, IOException {

	System.out.println(sAppName+"====> doGet was entered");

	POCDataBean dataBean;

	// Create an instance of DataBean
 
	try {
		dataBean = (POCDataBean) Beans.instantiate(this.getClass().getClassLoader(), "POCDataBean");
	} catch (Exception ex) {
		throw new ServletException("Can't create BEAN of class POCDataBean: "
                                       + ex);
	}


	dataBean.setTitle("CICS/Servlet Demo Code");
	dataBean.setServletPath(req.getServletPath());
	dataBean.setErrorText("");
	((com.sun.server.http.HttpServiceRequest) req).setAttribute("dataBean", dataBean);

	if (!(sConnectErrors.equals(""))) {
		dataBean.setErrorText("<P>Some init parameters are missing as indicated below.  The init parameters are specified when you define the servlet to WebSphere.  Please consult the documentation that came with this servlet if you don't know how to specify init parms.  You must resolve these errors before you can proceed.</P><UL>"+sConnectErrors+"</UL>");
		((com.sun.server.http.HttpServiceResponse) res).callPage("/IBMWebAS/POC/jsp/ErrorMessage.jsp", req);
		return;
	}
	
	//System.out.println(sAppName+"Just sent Header.jsp");


        //*******************************************************************************************************
	//*** if we have not and cannot connect to the CICS gateway for JAVA, put out error message
	if (!connectToGateway()) {
		dataBean.setTitle("POCServlet: Error Message");
		dataBean.setErrorText("The gateway to your application does not seems to be available, "+
			"please notify the helpdesk.<br>");
		((com.sun.server.http.HttpServiceResponse) res).callPage("/IBMWebAS/POC/jsp/ErrorMessage.jsp", req);
	} else {
		HttpSession session = req.getSession(true);                 // if a session does not exist, create one
		System.out.println(sAppName+"Getting session from doGet - Session:"+session);
		String UserName = (String) session.getValue("CICSECI.UserName"); // get the user name from the session
          	// if there is a user name, we have already authenticated them

		if ( UserName == null ) {
			((com.sun.server.http.HttpServiceResponse) res).callPage("/IBMWebAS/POC/jsp/SignonPanel.jsp", req);
		} else {
			((com.sun.server.http.HttpServiceResponse) res).callPage("/IBMWebAS/POC/jsp/FirstPanel.jsp", req);
		}
	}
	return;
    }  // end doGet()


    
    //**********************************************************************************************************
    //**********************************************************************************************************
    //*  process the POST method
    //**********************************************************************************************************
    //**********************************************************************************************************
    // ###8#
    public void doPost(HttpServletRequest req, HttpServletResponse res)
	          throws ServletException, IOException {

	ECIRequest eciRequest = null;					// ECI request object

        System.out.println(sAppName+"====> doPost entered");

	POCDataBean dataBean;

	// Create an instance of DataBean
	try {
		dataBean = (POCDataBean) Beans.instantiate(this.getClass().getClassLoader(), "POCDataBean");
	} catch (Exception ex) {
		throw new ServletException("Can't create BEAN of class POCDataBean: "
                                       + ex);
	}

	dataBean.setTitle("CICS/Servlet Demo Code");
	dataBean.setErrorText("");
	dataBean.setServletPath(req.getServletPath());
	((com.sun.server.http.HttpServiceRequest) req).setAttribute("dataBean", dataBean);
	
	System.out.println(sAppName+"Requesting pointer to Session information");

	HttpSession session = req.getSession(true);			// if a session does not exist, create one
	//System.out.println(sAppName+"Session:"+session);

	//**********************************************************************
	//*** if they pressed the logoff button, we should get a logoff=yes
	//**********************************************************************
        //  ###9#
	if (req.getParameter("logoff") != null) {
		System.out.println(sAppName+"Logging off");
		session.removeValue("CICSECI.UserName");		// remove the username from the session
		session.removeValue("CICSECI.PassWord");		// remove the password from the session
		((com.sun.server.http.HttpServiceResponse) res).callPage("/IBMWebAS/POC/jsp/SignonPanel.jsp", req);
		return;
	}

        //***********************************************************************
        //*** if the CICS Gateway for Java does not exist and we can not get a 
        //*** connection, leave a message and quit
        //***********************************************************************
        // ###10#
        //System.out.println(sAppName+"CICSGatewayConnection: "+CICSGatewayConnection);
	if (!connectToGateway()) {
		dataBean.setTitle("POCServlet: Error Message");
		dataBean.setErrorText("The gateway to your application does not seem to be available, "+
                        "please notify the helpdesk.<br>");
		((com.sun.server.http.HttpServiceResponse) res).callPage("/IBMWebAS/POC/jsp/ErrorMessage.jsp", req);
		return;
	} 

        //**************************************************************************
	//*** see what they want
	//**************************************************************************
	String signResp;				// signon response
	String userString;				// variable for user name
	String passString;				// variable for password
	String strSignCommarea;
	String UserName;
	byte[] bSignCommarea;

	UserName = (String)session.getValue("CICSECI.UserName");

	//***************************************************************************
	//*** if UserName is null, this must be a returned signon screen
	//*** go validate the userid and password
	//***************************************************************************
	// ###11#
	if ( UserName == null ) {
		System.out.println(sAppName+"session.UserName is null");
		//**************************************************
		//** prepare the commarea for a security check
		//**************************************************
		userString = req.getParameter("user")+"        ";
		userString = userString.substring(0,8);
		if (userString.equals("        ")) {
			((com.sun.server.http.HttpServiceResponse) res).callPage("/IBMWebAS/POC/jsp/SignonPanel.jsp", req);
			return;
		}
		passString = req.getParameter("pass")+"        ";
		passString = passString.substring(0,8);
		//                                                     1         2         3         4
		//                  2      8          8       1234567890123456789012345678901234567890  = 58
		strSignCommarea = "99"+userString+passString+"                                        ";
                bSignCommarea = strSignCommarea.getBytes();		// commarea must be bytes
		try {
			System.out.println(sAppName+"Trying ECI request for signon");
			eciRequest = new ECIRequest(strServerName,	// CICS Server
				userString,				// UserId, null for none
				passString,				// Password, null for none
				strSignProg,				// Program name
				bSignCommarea,				// Commarea, null for none
				ECIRequest.ECI_NO_EXTEND,		// single call in LUW
				ECIRequest.ECI_LUW_NEW);
			eciRequest.Cics_Rc = 0;
			CICSGatewayConnection.flow(eciRequest);	// flow the ECI request
				//out.println(" Return code: " + eciRequest.Cics_Rc+"<br>");
			
			if (eciRequest.Cics_Rc == 0) {
				System.out.println(sAppName+"Signon ECI request had an rc of 0");
				//out.println("The commarea:"+temp+"<br>");

				signResp = new String(bSignCommarea, 0, 2);
				if (signResp.equals("00")) {
					System.out.println(sAppName+"signResp was 0, storing userid and password");
					session.removeValue("CICSECI.UserName");
					session.putValue("CICSECI.UserName", userString);
					session.removeValue("CICSECI.PassWord");
					session.putValue("CICSECI.PassWord", passString);
					((com.sun.server.http.HttpServiceResponse) res).callPage("/IBMWebAS/POC/jsp/FirstPanel.jsp", req);
				} else {
					System.out.println(sAppName+"signResp = "+signResp);
					// send out bad signon message
					System.out.println(sAppName+"Signon ECI call ret code = "+
						eciRequest.getRc()+
						", "+
						eciRequest.Cics_Rc);
					dataBean.setErrorText("Note: your useridand password have been "+
						"determined to be an invalid combination.<br>");
					((com.sun.server.http.HttpServiceResponse) res).callPage("/IBMWebAS/POC/jsp/SignonPanel.jsp", req);
				}

			} else {
				// send out bad signon message
				System.out.println(sAppName+"Signon ECI call ret code = "+
					eciRequest.getRc()+
					", "+
					eciRequest.Cics_Rc);
				dataBean.setErrorText("Note: A problem has occured with the path to your application, "+
					"in the system that runs you application, or in the application itself.<br>"+
					"Please contact your Systems Support<br>");
				((com.sun.server.http.HttpServiceResponse) res).callPage("/IBMWebAS/POC/jsp/SignonPanel.jsp", req);
			}
		} catch (IOException e) {
			//out.println("Version 00.00.0011<br>");
			if (eciRequest == null) {
				dataBean.setErrorText("Unable to create eciRequest<br>");
			} else {
				dataBean.setErrorText("Exception Caught on Signon: Return code: " +
					eciRequest.Cics_Rc+"<br>");
			}
			((com.sun.server.http.HttpServiceResponse) res).callPage("/IBMWebAS/POC/jsp/ErrorMessage.jsp", req);
                }
		eciRequest = null;
		strSignCommarea = null;
		bSignCommarea = null;
		userString = null;
		passString = null;
		signResp = null;
		session = null;
		UserName = null;
		return;
	}

    //**********************************************************************************************************
    //*** if we got this far, this is a regular report request
    //**********************************************************************************************************
    // ###12#
	String passName;
	String userName;
	byte[] bTheCommarea;
	String reqID;
	String str;
		
	//((com.sun.server.http.HttpServiceResponse) res).callPage("/IBMWebAS/POC/jsp/FirstHTML.jsp", req);

	userName = (String) session.getValue("CICSECI.UserName"); // get user name from Cookie
	passName = (String) session.getValue("CICSECI.PassWord"); // get password from Cookie
	reqID = req.getParameter("reqid")+"      ";               // get parameters from HTML page
	reqID = reqID.substring(0,6);			  // ensure reqid is 6 positions
	System.out.println(sAppName+"Request was for: "+reqID+", as userid: "+userName+", password: "+passName);
        // prepare commarea
	//     1234    6    12345678901234567890   12345678901234567890  = 50
	str = "9999"+reqID+"                    "+"                    ";
	//out.println("length of str is "+str.length()+"<br>");
	// make ECI request
	try {
		//bTheCommarea = new byte[67];
		bTheCommarea = str.getBytes();
		eciRequest = new ECIRequest(strServerName,	// CICS Server
			userName,				// UserId, null for none
			passName,				// Password, null for none
			strRequProg,				// Program name
			bTheCommarea,				// Commarea, null for none
			ECIRequest.ECI_NO_EXTEND,
			ECIRequest.ECI_LUW_NEW);
		eciRequest.Cics_Rc = 0;
		CICSGatewayConnection.flow(eciRequest);		// Flow the request via the JGate to CICS
		//out.println(" Return code: " + eciRequest.Cics_Rc+"<br>");
		if (eciRequest.Cics_Rc == 0) {
			//String temp = new String(bTheCommarea);
			//out.println("The commarea:"+temp+"<br>");

			// put the returned data in a hash table
			dataBean.setReqID(reqID);
			dataBean.setFirstStr(new String(bTheCommarea, 10, 20));
			dataBean.setLastStr(new String(bTheCommarea, 30, 20));
			((com.sun.server.http.HttpServiceResponse) res).callPage("/IBMWebAS/POC/jsp/LastPanel.jsp", req);
		} else {
			System.out.println(sAppName+"Signon ECI call ret code = "+eciRequest.getRc());
			dataBean.setErrorText("Note: A Problem has occured with the path to your application, "+
				"in the system that runs your application, or in the application itself.<br>"+
				"Please contact your Systems Support<br>"+
				"Code1 = "+eciRequest.Cics_Rc+", Code2 = "+eciRequest.Abend_Code);
			((com.sun.server.http.HttpServiceResponse) res).callPage("/IBMWebAS/POC/jsp/ErrorMessage.jsp", req);
		}
	} 
	catch (IOException e) {
		dataBean.setErrorText("Exception Caught: Return code: " + eciRequest.Cics_Rc+"<br>");
		((com.sun.server.http.HttpServiceResponse) res).callPage("/IBMWebAS/POC/jsp/ErrorMessage.jsp", req);
	}
	return;

    }   // end doPost


    //*************************************************************************************************************
    //*************************************************************************************************************
    //*** destroy the Gateway connection when the servlet is unloaded
    //*************************************************************************************************************
    //*************************************************************************************************************
    // ###13#
    public void destroy() {
	System.out.println(sAppName+"destroy() method entered.");
	try {
		if (CICSGatewayConnection != null) {
			CICSGatewayConnection.close();
			CICSGatewayConnection = null;
		}
	}
	catch (IOException eClose) {
	}
    }  //end of destroy




    //************************************************************************************************************
    //************************************************************************************************************
    //* get servlet info
    //************************************************************************************************************
    //************************************************************************************************************
    public String getServletInfo() {
        return "The POCServlet servlet";
    }  // end getServletInfo



} // end POCServlet class
